package com.hncy.datatrans.server.pzhdata.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.hncy.datatrans.server.pzhdata.entity.ConflictDisputeResolve;
import com.hncy.datatrans.server.pzhdata.mapper.ConflictDisputeResolveMapper;
import com.hncy.datatrans.server.pzhdata.utils.IvanTool;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author Ivan
 * @Description 矛盾纠纷excel监听器
 * @Date 2020/8/25 11:13
 */
@Slf4j
public class ConflictExcelListener extends AnalysisEventListener<ConflictExcelData> {

    /**
     * 每隔1000条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 500;

    //因为ExcelListener不能交给spring进行管理，需要自己new，不能注入其他对象 不能实现数据库操作
    public ConflictDisputeResolveMapper conflictMapper;

    //存储数据
    List<ConflictExcelData> dataList = new ArrayList<>();

    //无参构造函数
    public ConflictExcelListener() {}
    //有参构造函数
    public ConflictExcelListener(ConflictDisputeResolveMapper conflictMapper) {
        this.conflictMapper = conflictMapper;
    }

    //读取excel内容，一行一行进行读取
    @Override
    public void invoke(ConflictExcelData conflictExcelData, AnalysisContext analysisContext) {
        if(conflictExcelData == null) {
            System.out.println("文件数据为空...");
            throw new RuntimeException("文件数据为空...");
        }
//        System.out.println(conflictExcelData);
        dataList.add(conflictExcelData);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (dataList.size() >= BATCH_COUNT) {
            //保存到数据库
            saveConflict();
            // 存储完成清理 list
            dataList.clear();
        }
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveConflict();
        log.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveConflict() {
        //日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        log.info("{}条数据，开始存储数据库！", dataList.size());
        //处理数据
        if (dataList!=null&&dataList.size()>0){
            List<ConflictDisputeResolve> conflicts = new ArrayList<>();
            for (ConflictExcelData data : dataList){
                ConflictDisputeResolve conflict = new ConflictDisputeResolve();

                //复制相同的字段
                BeanUtils.copyProperties(data,conflict);

                //主键id
                conflict.setId(IvanTool.getId());
                //档案编号
                String fileNoData = data.getFileNo();
                if (fileNoData!=null){
                    conflict.setFileNo(fileNoData.substring(fileNoData.indexOf("：") + 1));
                }

                try {
                    //处理特殊类型字段：辖区、Date、int等
                    if (data.getHappenDate()!=null){
                        Date happenDate = sdf.parse(data.getHappenDate());
                        conflict.setHappenDate(happenDate);
                    }

                    //名称
                    conflict.setEventName(data.getEventType());

                    //涉及人数
                    if (data.getInvolvePersonNumber()!=null){
                        conflict.setInvolvePersonNumber(Integer.parseInt(data.getInvolvePersonNumber()));
                    }
                    //化解是否成功
                    if (data.getIsResolveSuccess()!=null){
                        int success = data.getIsResolveSuccess().contains("成功") ? 1 : 0;
                        conflict.setIsResolveSuccess(success);
                    }

                    //辖区
                    String address = data.getHappenAddress();
                    if (address!=null){
                        String areaId = null;
                        String areaName = null;
                        if (address.contains("东区")){
                            areaId = "2";
                            areaName = "东区";
                        }
                        if (address.contains("西区")){
                            areaId = "3";
                            areaName = "西区";
                        }
                        if (address.contains("仁和区")){
                            areaId = "4";
                            areaName = "仁和区";
                        }
                        if (address.contains("米易县")){
                            areaId = "5";
                            areaName = "米易县";
                        }
                        if (address.contains("盐边县")){
                            areaId = "6";
                            areaName = "盐边县";
                        }
                        if (address.contains("钒钛")){
                            areaId = "7";
                            areaName = "钒钛新城";
                        }
                        conflict.setAreaId(areaId);
                        conflict.setAreaName(areaName);
                    }


                    //纠纷类别
                    String eventType = data.getEventType();
                    if (eventType!=null){
                        if (eventType.contains("婚姻")){
                            conflict.setEventTypeCode("217065");
                        } else if (eventType.contains("邻里")){
                            conflict.setEventTypeCode("217073");
                        } else if (eventType.contains("人格")){
                            conflict.setEventTypeCode("217078");
                        } else if (eventType.contains("物权")){
                            conflict.setEventTypeCode("217083");
                        } else if (eventType.contains("侵权")){
                            conflict.setEventTypeCode("217088");
                        } else if (eventType.contains("合同")){
                            conflict.setEventTypeCode("217097");
                        } else if (eventType.contains("劳动")){
                            conflict.setEventTypeCode("217103");
                        } else if (eventType.contains("金融")){
                            conflict.setEventTypeCode("217109");
                        } else if (eventType.contains("土地")){
                            conflict.setEventTypeCode("217115");
                        } else if (eventType.contains("民商")){
                            conflict.setEventTypeCode("217121");
                        } else if (eventType.contains("城乡")){
                            conflict.setEventTypeCode("217129");
                        } else if (eventType.contains("民政")){
                            conflict.setEventTypeCode("217136");
                        } else if (eventType.contains("行政")){
                            conflict.setEventTypeCode("217141");
                        } else if (eventType.contains("诉讼")){
                            conflict.setEventTypeCode("217150");
                        } else if (eventType.contains("组织")){
                            conflict.setEventTypeCode("217155");
                        } else if (eventType.contains("民族")){
                            conflict.setEventTypeCode("217159");
                        } else {
//                          conflict.setEventType("其他");
                            conflict.setEventTypeCode("217162");
                        }
                    }

                } catch (Exception e){
                    e.printStackTrace();
                }

                //添加一条数据
                conflicts.add(conflict);
            }
            int result = conflictMapper.saveConflict(conflicts);
            log.info(result + "条数据保存成功！");
        }
    }
}
